package cn.iocoder.yudao.module.bpm.service.oa.entry;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.IdUtil;
import cn.iocoder.yudao.module.bpm.api.task.BpmProcessInstanceApi;
import cn.iocoder.yudao.module.bpm.api.task.dto.BpmProcessInstanceCreateReqDTO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.entry.vo.OaEntryPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.entry.vo.OaEntrySaveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskApproveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskReturnReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.entry.OaEntryDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.entry.OaEntryMapper;
import cn.iocoder.yudao.module.bpm.service.task.BpmTaskService;
import cn.iocoder.yudao.module.infra.api.file.dto.FileDTO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import org.apache.poi.openxml4j.opc.PackagingURIHelper;
import org.flowable.engine.RuntimeService;
import org.flowable.engine.TaskService;
import org.flowable.engine.runtime.ProcessInstance;
import org.flowable.task.api.Task;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;

import java.util.*;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;


import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.*;

/**
 * 入职流程信息 Service 实现类
 *
 * @author 芋道源码
 */
@Service
@Validated
public class OaEntryServiceImpl implements OaEntryService {

    /**
     * OA 入职对应的流程定义 KEY
     */
    public static final String PROCESS_KEY = "oa_entry_workflow";

    @Resource
    private OaEntryMapper oaEntryMapper;

    @Resource
    private BpmProcessInstanceApi processInstanceApi;

    @Resource
    private BpmTaskService bpmTaskService;

    @Resource
    private TaskService taskService;

    @Autowired
    private RuntimeService runtimeService;

    @Override
    public Integer startOaEntry(Long userId, OaEntrySaveReqVO createReqVO) {
        //判断是否已经发起流程
        Boolean auditPass = createReqVO.getAuditPass();
        //创建单据uuid
        String uuid = IdUtil.fastSimpleUUID();
        // 插入
        OaEntryDO oaEntry = BeanUtils.toBean(createReqVO, OaEntryDO.class);
        oaEntryMapper.insert(oaEntry);
        //如果提交
        if (auditPass){
            // 发起 BPM 流程
            Map<String, Object> processInstanceVariables = new HashMap<>();
            processInstanceVariables.put("auditPass",auditPass);
            //添加审批人信息
            Map<String, List<Long>> startUserSelectAssignees = new HashMap<>();
            startUserSelectAssignees.put("approver",createReqVO.getStartUserSelectAssignees());
            String processInstanceId = processInstanceApi.createProcessInstance(userId,
                    new BpmProcessInstanceCreateReqDTO().setProcessDefinitionKey(PROCESS_KEY)
                            .setVariables(processInstanceVariables).setBusinessKey(String.valueOf(oaEntry.getId()))
                            .setStartUserSelectAssignees(startUserSelectAssignees));

            //获取下一个审批人
            Task task = taskService.createTaskQuery().processInstanceId(processInstanceId).singleResult();

            // 将工作流的编号，单据状态 表单uuid  最后一次审批时间 当前审批人  更新到 OA 入职中
            oaEntryMapper.updateById(new OaEntryDO().setId(oaEntry.getId()).setProcInstId(processInstanceId).setAuditStatus("1")
                    .setEntryId(uuid).setFinalAuditDate(new Date()).setCurrentAuditEmployeeId(task.getAssignee()));
        }else {
            // 将单据状态 表单uuid 更新到 OA入职中
            oaEntryMapper.updateById(new OaEntryDO().setId(oaEntry.getId()).setAuditStatus("0").setEntryId(uuid));
        }

        //保存附件uuid
        if (CollectionUtil.isNotEmpty(createReqVO.getFileIdList())) {
            LambdaUpdateWrapper<FileDTO> wrapper = new LambdaUpdateWrapper<>();
            wrapper.in(FileDTO::getId, createReqVO.getFileIdList()).set(FileDTO::getServiceId, uuid);
        }
        // todo 站内信发送
        // loanInfoService.sendMassage(actBaseInfo);
        // 返回
        return oaEntry.getId();
    }

    @Override
    public Integer completeOaEntry(Long userId, OaEntrySaveReqVO createReqVO, BpmTaskApproveReqVO reqVO) {
        //判断是否已经发起流程
        Boolean auditPass = createReqVO.getAuditPass();
        // 修改
        OaEntryDO oaEntry = BeanUtils.toBean(createReqVO, OaEntryDO.class);
//        oaEntryMapper.updateById(oaEntry);
        //判断当前单据是否存在流程实例id，如果不存在说明没有发起流程
        if (oaEntry.getProcInstId() == null && oaEntry.getProcInstId().equals("")) {
            // 提交
            if (createReqVO.getAuditPass()) {
                // 发起 BPM 流程
                Map<String, Object> processInstanceVariables = new HashMap<>();
                processInstanceVariables.put("auditPass", auditPass);
                //添加审批人信息
                Map<String, List<Long>> startUserSelectAssignees = new HashMap<>();
                startUserSelectAssignees.put("approver",createReqVO.getStartUserSelectAssignees());
                String processInstanceId = processInstanceApi.createProcessInstance(userId,
                        new BpmProcessInstanceCreateReqDTO().setProcessDefinitionKey(PROCESS_KEY)
                                .setVariables(processInstanceVariables).setBusinessKey(String.valueOf(oaEntry.getId()))
                                .setStartUserSelectAssignees(startUserSelectAssignees));

                // 将工作流的编号，单据状态 更新到 OA 入职中
                oaEntryMapper.updateById(new OaEntryDO().setId(oaEntry.getId()).setProcInstId(processInstanceId).setAuditStatus("1"));
            }else {
                // 暂存 将单据状态 更新到 OA 入职中
                oaEntryMapper.updateById(new OaEntryDO().setId(oaEntry.getId()).setAuditStatus("0"));
            }
        } else {
            //同意
            if (createReqVO.getAuditPass()) {
                //审批通过
                bpmTaskService.approveTask(userId,reqVO);
                //判断流程是否结束
                ProcessInstance procIns = runtimeService.createProcessInstanceQuery().processInstanceId(createReqVO.getProcInstId()).active().singleResult();
                if (procIns != null) {
                    //说明流程结束
                    //获取下一个审批人
                    Task task = taskService.createTaskQuery().processInstanceId(createReqVO.getProcInstId()).singleResult();
                    //  将单据状态 当前审批人 最后一次审批时间 更新到 OA 入职中
                    oaEntryMapper.updateById(new OaEntryDO().setId(oaEntry.getId()).setAuditStatus("2").setCurrentAuditEmployeeId(task.getAssignee())
                            .setFinalAuditDate(new Date()));
                }else {
                    //流程结束
                    //todo 复制信息到入职信息表中

                    //  将单据状态 更新到 OA 入职中
                    oaEntryMapper.updateById(new OaEntryDO().setId(oaEntry.getId()).setAuditStatus("3").setCurrentAuditEmployeeId(" ").setFinalAuditDate(new Date()));
                }
            }else {
//                //审批不通过
//                BpmTaskReturnReqVO bpmTaskReturnReqVO =new BpmTaskReturnReqVO();
//                        bpmTaskReturnReqVO.setId(reqVO.getId());
//                        bpmTaskReturnReqVO.setReason(reqVO.getReason());
//                        bpmTaskReturnReqVO.setTargetTaskDefinitionKey("");
//                bpmTaskService.returnTask(userId,reqVO);
//                //  将单据状态 当前审批人 最后一次审批时间 更新到 OA 入职中
//                oaEntryMapper.updateById(new OaEntryDO().setId(oaEntry.getId()).setAuditStatus("4").setCurrentAuditEmployeeId(" ").setFinalAuditDate(new Date()));
            }
        }

        //保存附件uuid
        LambdaUpdateWrapper<FileDTO> wrapper = new LambdaUpdateWrapper<>();
        wrapper.in(FileDTO::getId, createReqVO.getFileIdList()).set(FileDTO::getServiceId, createReqVO.getEntryId());

        // todo 站内信发送
        // loanInfoService.sendMassage(actBaseInfo);
        // 返回
        return oaEntry.getId();
    }

    @Override
    public void updateOaEntry(OaEntrySaveReqVO updateReqVO) {
        // 校验存在
        validateOaEntryExists(updateReqVO.getId());
        // 更新
        OaEntryDO updateObj = BeanUtils.toBean(updateReqVO, OaEntryDO.class);
        oaEntryMapper.updateById(updateObj);
    }

    @Override
    public void deleteOaEntry(Integer id) {
        // 校验存在
        validateOaEntryExists(id);
        // 删除
        oaEntryMapper.deleteById(id);
    }

    private void validateOaEntryExists(Integer id) {
        if (oaEntryMapper.selectById(id) == null) {
            throw exception(OA_ENTRY_NOT_EXISTS);
        }
    }

    @Override
    public OaEntryDO getOaEntry(Integer id) {
        return oaEntryMapper.selectById(id);
    }

    @Override
    public PageResult<OaEntryDO> getOaEntryPage(OaEntryPageReqVO pageReqVO) {
        return oaEntryMapper.selectPage(pageReqVO);
    }

}